@@ -6,8 +6,10 @@ use JSON;
 use Mouse;
 use Net::OAuth;
 use LWP::UserAgent;
+use URI;
 use HTTP::Request::Common;
 use Data::Random qw(rand_chars);
+use Encode;
 
 =head1 NAME
 
@@ -15,11 +17,11 @@ Net::Dropbox::API - A dropbox API interface
 
 =head1 VERSION
 
-Version 1.0.0.10.10.9.8
+Version 1.6.1.6.5.4.3
 
 =cut
 
-our $VERSION = '1.0';
+our $VERSION = '1.6.1';
 
 
 =head1 SYNOPSIS
@@ -31,13 +33,15 @@ This is how it works:
     use Net::Dropbox::API;
 
     my $box = Net::Dropbox::API->new({key => 'KEY', secret => 'SECRET'});
-    my $login_link = $box->login;  # user needs to klick this link and login
+    my $login_link = $box->login;  # user needs to click this link and login
     $box->auth;                    # oauth keys get exchanged
     my $info = $box->account_info; # and here we have our account info
 
 See the examples for a working Mojolicious web client using the Dropbox
 API.
 
+You can find Dropbox's API documentation at L<https://www.dropbox.com/developers/web_docs>
+
 =head1 FUNCTIONS
 
 =cut
@@ -58,7 +62,7 @@ has 'context' => (is => 'rw', isa => 'Str', default => 'sandbox');
 =head2 login
 
 This sets up the initial OAuth handshake and returns the login URL. This
-URL has to be clicked by the user and the the user then has to accept
+URL has to be clicked by the user and the user then has to accept
 the application in dropbox. 
 
 Dropbox then redirects back to the callback URL defined with
@@ -75,12 +79,13 @@ sub login {
     my $request = Net::OAuth->request("request token")->new(
         consumer_key => $self->key,
         consumer_secret => $self->secret,
-        request_url => 'http://api.dropbox.com/0/oauth/request_token',
+        request_url => 'https://api.dropbox.com/0/oauth/request_token',
         request_method => 'POST',
         signature_method => 'HMAC-SHA1',
         timestamp => time,
         nonce => $self->nonce,
         callback => $self->callback_url,
+        callback_confirmed => ($self->callback_url ? 'true' : undef)
     );
 
     $request->sign;
@@ -92,7 +97,7 @@ sub login {
         $self->request_secret($response->token_secret);
         print "Got Request Token ", $response->token, "\n" if $self->debug;
         print "Got Request Token Secret ", $response->token_secret, "\n" if $self->debug;
-        return 'http://api.dropbox.com/0/oauth/authorize?oauth_token='.$response->token.'&oauth_callback='.$self->callback_url;
+        return 'https://www.dropbox.com/0/oauth/authorize?oauth_token='.$response->token.'&oauth_callback='.$self->callback_url;
     }
     else {
         $self->error($res->status_line);
@@ -115,7 +120,7 @@ sub auth {
     my $request = Net::OAuth->request("access token")->new(
         consumer_key => $self->key,
         consumer_secret => $self->secret,
-        request_url => 'http://api.dropbox.com/0/oauth/access_token',
+        request_url => 'https://api.dropbox.com/0/oauth/access_token',
         request_method => 'POST',
         signature_method => 'HMAC-SHA1',
         timestamp => time,
@@ -155,15 +160,67 @@ sub account_info {
 
 =head2 list
 
-lists all files in the path defined.
+lists all files in the path defined:
+
+    $data = $box->list();           # top-level
+    $data = $box->list( "/Photos" ); # folder
+
+The data returned is a ref to a hash containing various fields returned
+by Dropbox, including a C<hash> value, which can be used later to check
+if Dropbox data beneath a specified folder has changed since the last call.
+
+For this, C<list()> accepts an optional 'hash' argument:
+
+    $data = $box->list({ hash => "ce9ccbfb8f255f234c93adcfef33b5a6" },
+                       "/Photos");
+
+This will either return
+
+    { http_response_code => 304 }
+
+in which case nothing has changed since the last call, or 
+
+    { http_response_code => 200,
+      # ... various other fields
+    }
+
+if there were modifications.
 
 =cut
 
 sub list {
     my $self = shift;
-    my $path = shift || '';
+    my $opts = {};
+    if(defined $_[0]  and ref($_[0]) eq "HASH") {
+          # optional option hash present
+        $opts = shift;
+    }
+    my $path = shift;
+    $path = '' unless defined $path;
+    $path = '/'.$path if $path=~m|^[^/]|;
 
-    return from_json($self->_talk('files/'.$self->context.'/'.$path));
+    my $uri = URI->new('files/'.$self->context.$path);
+    $uri->query_form($opts) if scalar keys %$opts;
+
+    my $talk_opts = {};
+
+    if(exists $opts->{hash}) {
+       $talk_opts = {
+           error_handler => sub {
+               my $obj   = shift;
+               my $resp  = shift;
+               # HTTP::Status is nice but old RHEL5 has issues with it
+               # so we use plain codes
+               if( $resp->code == 304 ) {
+                   return to_json({ http_response_code => 304 });
+               } else {
+                   return $self->_talk_default_error_handler($resp);
+               }
+           },
+       };
+    }
+
+    return from_json($self->_talk($talk_opts, $uri->as_string));
 }
 
 =head2 copy
@@ -177,8 +234,10 @@ sub copy {
     my $self = shift;
     my ($from, $to) = @_;
 
-    my $opts = 'from_path='.$from.'&to_path='.$to.'&root='.$self->context;
-    return from_json($self->_talk('fileops/copy?'.$opts));
+    my $opts = 'root='.$self->context;
+    return from_json($self->_talk('fileops/copy?'.$opts,
+                    undef, undef, undef, undef, undef,
+                    { from_path => $from, to_path => $to }));
 }
 
 =head2 move
@@ -192,8 +251,10 @@ sub move {
     my $self = shift;
     my ($from, $to) = @_;
 
-    my $opts = 'from_path='.$from.'&to_path='.$to.'&root='.$self->context;
-    return from_json($self->_talk('fileops/move?'.$opts));
+    my $opts = 'root='.$self->context;
+    return from_json($self->_talk('fileops/move?'.$opts,
+                    undef, undef, undef, undef, undef,
+                    { from_path => $from, to_path => $to }));
 }
 
 =head2 mkdir
@@ -207,8 +268,10 @@ sub mkdir {
     my $self = shift;
     my ($path) = @_;
 
-    my $opts = 'path='.$path.'&root='.$self->context;
-    return from_json($self->_talk('fileops/create_folder?'.$opts));
+    my $opts = 'root='.$self->context;
+    return from_json($self->_talk('fileops/create_folder?'.$opts,
+                    undef, undef, undef, undef, undef,
+                    { path => $path }));
 }
 
 =head2 delete
@@ -222,8 +285,10 @@ sub delete {
     my $self = shift;
     my ($path) = @_;
 
-    my $opts = 'path='.$path.'&root='.$self->context;
-    return from_json($self->_talk('fileops/delete?'.$opts));
+    my $opts = 'root='.$self->context;
+    return from_json($self->_talk('fileops/delete?'.$opts,
+                    undef, undef, undef, undef, undef,
+                    { path => $path }));
 }
 
 =head2 view
@@ -271,12 +336,36 @@ sub putfile {
             'files/'.$self->context.'/'.$path,
             'POST',
             { file => [ $file ] },
-            $filename
+            $filename, # can't decode_utf8
+            'api-content',
+            undef,
+            { file => decode_utf8($filename) }
         )
     );
 
 }
 
+=head2 getfile
+
+get a file from dropbox
+
+=cut
+
+=head2 debug
+
+Set this to a non-false value in order to print some debugging information to STDOUT.
+    debug(1)
+
+=cut
+
+sub getfile {
+    my $self = shift;
+    my $path = shift || '';
+    my $file = shift || '';
+
+    return $self->_talk('files/'.$self->context.'/'.$path, undef, undef, undef, 'api-content', $file);
+}
+
 
 =head1 INTERNAL API
 
@@ -297,17 +386,29 @@ sub nonce { join( '', rand_chars( size => 16, set => 'alphanumeric' )); }
 
 sub _talk {
     my $self    = shift;
+    my $opts    = {};
+    if(defined $_[0]  and ref($_[0]) eq "HASH") {
+          # optional option hash present
+        $opts = shift;
+    }
     my $command = shift;
     my $method  = shift || 'GET';
     my $content = shift;
     my $filename= shift;
+    my $api     = shift || 'api';
+    my $content_file = shift;
+    my $extra_params = shift;
+
+    if( !defined $opts->{error_handler} ) {
+        $opts->{error_handler} = \&_talk_default_error_handler;
+    }
 
     my $ua = LWP::UserAgent->new;
 
     my %opts = (
         consumer_key => $self->key,
         consumer_secret => $self->secret,
-        request_url => 'http://api.dropbox.com/0/'.$command,
+        request_url => 'https://'.$api.'.dropbox.com/0/'.$command,
         request_method => $method,
         signature_method => 'HMAC-SHA1',
         timestamp => time,
@@ -315,11 +416,10 @@ sub _talk {
         #callback => $self->callback_url,
         token => $self->access_token,
         token_secret => $self->access_secret,
+        extra_params => $extra_params
     );
     if($filename) {
         push @{$content->{file}},$filename;
-        $opts{request_url} = "http://api-content.dropbox.com/0/$command/?".
-            "file=$filename";
     }
 
     my $request = Net::OAuth->request("protected resource")->new( %opts );
@@ -327,7 +427,9 @@ sub _talk {
     $request->sign;
 
     my $res;
-    if($method =~ /get/i){
+    if($content_file) {
+        $res = $ua->get($request->to_url, ':content_file' => $content_file);
+    } elsif($method =~ /get/i){
         $res = $ua->get($request->to_url);
     } else {
         $res = $ua->post($request->to_url, Content_Type => 'form-data', Content => $content );
@@ -335,31 +437,50 @@ sub _talk {
 
     if ($res->is_success) {
         print "Got Content ", $res->content, "\n" if $self->debug;
-        return $res->content;
-    }
-    else {
+        my $data;
+        eval {
+            $data = from_json($res->content);
+        };
+        if($@) {
+            # this doesn't look like JSON, might be file content
+            return $res->content;
+        }
+        $data->{http_response_code} = $res->code();
+        return to_json($data);
+    } else {
         $self->error($res->status_line);
-        warn "Something went wrong: ".$res->status_line;
+        return $opts->{error_handler}->($self, $res);
     }
     return;
 }
 
-=head2 talk
+sub _talk_default_error_handler {
+    my $self    = shift;
+    my $res     = shift;
 
-=cut
+    warn "Something went wrong: ".$res->status_line;
+    return to_json({error => $res->status_line,
+                    http_response_code => $res->code});
+}
 
 =head1 AUTHOR
 
 Lenz Gschwendtner, C<< <norbu09 at cpan.org> >>
 
 With Bug fixes from:
+
 Greg Knauss C<< gknauss at eod.com >>
+
 Chris Prather C<< chris at prather.org >>
 
+Shinichiro Aska
+
+[ktdreyer]
+
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-net-dropbox-api at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-Dropbox-API>.  I will be notified, and then you'll
+Please report any bugs through the web interface at
+L<https://github.com/norbu09/Net--Dropbox/issues>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 =head1 SUPPORT
@@ -372,10 +493,6 @@ You can also look for information at:
 
 =over 4
 
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-Dropbox-API>
-
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Net-Dropbox-API>
